#ifndef STONE_H
#define STONE_H

/** Copyright (C) 2010 Maxime Gamboni. See COPYING for
    copying/warranty info. */

#define CLS_STONE 1

/* Used when push()ing a stone over the network */
struct stone_info {
  int id; /* item identifier as returned by Level::register_item */
  unsigned char cls; /* must be CLS_STONE */
  unsigned char ev; /* EV_something */
  char ch; /* stone character like in the level files */
  int x;
  int y;
};

class Stone;

#include "item.h"
#include "cell.h"
#include "littlecube.h"

class Stone : public Item {
 public:

  Stone(unsigned int id);
  /* Initialise the stone. The character tells what letter was used in
     the level description */
  virtual void init(char ch,int x,int y);
  virtual ~Stone();

  void poll();
  void process(struct item_info *data,size_t len);

  /* The default behaviour is to always be at the position passed to
     init(). If you override this method the x, y instance variables
     are no longer used, and you may want to override check_collision
     to set the v parameter accordingly.  */
  virtual double getX();
  virtual double getY();

  virtual void render();

  /* This just renders a cube without setting colours or textures or
     anything. Can be used by replacement render()s. Also returns true
     if the stone is still intact in case you want to render stuff
     around it. */
  bool render0();

  /* Notify the camera that something interesting is happening over here */
  void lookAtMe(double t);

  /* Check if the segment (x;y) -> (x+dx;y+dy) intersects this.  If it
     does, returns a number between zero and one indicating at which
     point the line (first) intersected (zero=beginning, one=end) and
     store the normal vector into n, and store zero into v (that is,
     the stone speed, but stones don't move)

     Otherwise return 2.

     (n and v must be pointers to double arrays of two elements).

     The trigger parameter is only used for stones that present
     different shapes depending what's going to hit them (i.e. this
     method does *not* call trigger even if triggers is set to true)
  */
  virtual double check_collision(double x,double y,double dx,double dy, double *n, double *v,bool triggers=false);

  /* Return true if the stone must be broken in order for the level to
     be passed. Do NOT make this return false if trigger may ever
     return true or stone counting will get messed up. */
  virtual bool breakable();

  /** Call this to notify the stone it was hit by something (when this
      is called you can assume that the object was following the
      course described by the latest checkCollision).

      Returns true if this step killed the stone.

      Pass null if this wasn't hit by a littlecube. */
  virtual bool trigger(Littlecube *b);

  /** Request the stone to perform its associated action, if any
      When the user clicks, all stones are "activated". For most
      stones this doesn't do anything. */
  virtual void activate();

  /* break this stone. This should only be called *from* trigger
     unless you want to force the stone to break. Set the parameter
     to true if you want the pieces themselves to trigger other
     stones. */
  void kill(bool piecetriggers=false);

  /* The parameter tells how much time passed since the last update() */
  virtual void update(double t, Level *lvl);

 protected:

  char ch;
  int x;
  int y;

  //  bool interesting; // whether this should be looked at

 private:

  bool gone; // rather than removing a stone when it breaks we just
	     // set its "gone" flag to true.
  Littlecube **pieces;

};

/** Create a stone for the corresponding character. Returns NULL if
    ch==' ' or if it's not a known type of stone. */
Stone *new_stone(char ch,int id=0);

#endif
